unit UProperties;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, ToolEdit, ComCtrls, Buttons, ExtCtrls, Menus;

const ConfigName='fezinc.ini';

type
  tDriver=(drvKeyboard,drvRenderer,drvSound,drvNone);
  TfrmProperties = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    mmoSetup: TMemo;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    tsRenderer: TTabSheet;
    gbRenderer: TGroupBox;
    cbRenderer: TComboBox;
    GroupBox1: TGroupBox;
    edCutOff: TEdit;
    GroupBox2: TGroupBox;
    edCBuffer: TEdit;
    GroupBox3: TGroupBox;
    edDBuffer: TEdit;
    cbInterpreter: TCheckBox;
    cbGeometry: TCheckBox;
    GroupBox4: TGroupBox;
    edDir: TDirectoryEdit;
    cbSound: TCheckBox;
    cbSoundFilter: TCheckBox;
    cbStereo: TCheckBox;
    cbSurround: TCheckBox;
    GroupBox5: TGroupBox;
    edSurrMult: TEdit;
    cbWindow: TCheckBox;
    SpeedButton1: TSpeedButton;
    Label4: TLabel;
    edXMax: TEdit;
    edYMax: TEdit;
    cbDithering: TCheckBox;
    Label6: TLabel;
    cbDisplay: TComboBox;
    Label7: TLabel;
    cbBlending: TComboBox;
    Label8: TLabel;
    cbScanlines: TComboBox;
    Image1: TImage;
    lblFilename2: TLabel;
    Bevel1: TBevel;
    GroupBox6: TGroupBox;
    cbBPP: TComboBox;
    GroupBox7: TGroupBox;
    cbFPS: TCheckBox;
    cbSkip: TCheckBox;
    cbLimit: TCheckBox;
    cbDetect: TCheckBox;
    GroupBox8: TGroupBox;
    Label1: TLabel;
    cbTFiltering: TComboBox;
    cbTCaching: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    cbTQuality: TComboBox;
    lblFilename1: TLabel;
    Image2: TImage;
    Bevel2: TBevel;
    pmResolution: TPopupMenu;
    N320x2001: TMenuItem;
    N640x4801: TMenuItem;
    N800x6001: TMenuItem;
    N1024x7681: TMenuItem;
    N1400x12001: TMenuItem;
    Label5: TLabel;
    edFRate: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure N320x2001Click(Sender: TObject);
    procedure N640x4801Click(Sender: TObject);
    procedure N800x6001Click(Sender: TObject);
    procedure N1024x7681Click(Sender: TObject);
    procedure N1400x12001Click(Sender: TObject);
    procedure editorKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
    fFilename:String;
    procedure Convert(Input:String;Output:String='renderer.znc');
    procedure SetFileName(Value:String);
    function GetRenderer:String;
    function DLLType(Filename:String):TDriver;
  public
    { Public-Deklarationen }
    procedure LoadSettings(Nr:Integer);
    procedure SaveSettings(Nr:Integer);
    procedure CreateConfig;
    property Filename:String read FFilename write SetFilename;
    property Renderer:String read GetRenderer;
  end;

var
  frmProperties: TfrmProperties;

implementation uses shellapi,inifiles;

{$R *.DFM}

procedure WriteDebug(S:String);
var f:textfile;
begin
 assignfile(f,'debug.txt');
 if fileexists('debug.txt') then append(f) else rewrite(f);
 writeln(f,s);
 close(f);
end;

function TfrmProperties.DLLType(Filename:String):TDriver;
var lib:Cardinal;
begin
 lib := LoadLibrary(Pchar(Filename));
 if (lib <> INVALID_HANDLE_VALUE) and (lib<>0) then begin
  if GetProcAddress(lib,'ZN_JammaOpen')<>nil then result:=DrvKeyboard else
  if GetProcAddress(lib,'ZN_GPUinit')<>nil then result:=DrvRenderer else
  if GetProcAddress(lib,'ZN_SPUopen')<>nil then result:=DrvSound else
   result:=DrvNone;
 end;
 freelibrary(lib);
end;

function TfrmProperties.GetRenderer:string;
var dir:string;
begin
 dir:=extractfilepath(Application.ExeName);
 if cbRenderer.text='' then result:='' else result:=dir+cbRenderer.text;
end;

procedure TfrmProperties.SetFileName(Value:String);
begin
 fFilename:=Value;
 lblFilename1.Caption:=value;
 lblFilename2.Caption:=value;
// lblFilename3.Caption:=Value;
end;

procedure TfrmProperties.LoadSettings(Nr:Integer);
var ini:Tinifile;
    fname:String;
    rom:string;
    index:integer;
begin
 try
  fname:=extractfilepath(Application.ExeName)+'\'+configName;
  if not fileexists(fname) then mmoSetup.Lines.SaveToFile(fname);

  rom:='Rom'+inttostr(nr);
  ini:=TInifile.Create(fname);
  if nr>0 then begin
   edCutOff.text:=ini.ReadString(Rom,'CutOff','22050');
   edCBuffer.text:=ini.ReadString(Rom,'CBuffer','4096');
   edDBuffer.text:=ini.ReadString(Rom,'DBuffer','2048');
   cbInterpreter.Checked:=ini.ReadBool(Rom,'Interpreter',True);
   cbGeometry.Checked:=ini.ReadBool(Rom,'Geometry',True);
   cbSound.Checked:=ini.ReadBool(Rom,'Sound',True);
   cbSoundFilter.checked:=ini.ReadBool(Rom,'Filter',False);
   cbSurround.checked:=ini.ReadBool(Rom,'Surround',False);
   cbStereo.Checked:=ini.ReadBool(Rom,'Stereo',False);
   edSurrMult.Text:=ini.ReadString(Rom,'Multiplier','40');
   index:=ini.ReadInteger(Rom,'Renderer',0);
   if cbRenderer.Items.count>=index then
    cbRenderer.ItemIndex:=index else
    cbRenderer.ItemIndex:=-1;

   // Renderer
    cbWindow.Checked:=ini.ReadBool(Rom,'Windowed',true);
    edXmax.Text:=ini.ReadString(Rom,'Xmax','640');
    edYmax.Text:=ini.ReadString(Rom,'Ymax','480');
    cbDithering.Checked:=ini.ReadBool(Rom,'Dithering',false);
    cbBpp.ItemIndex:=ini.ReadInteger(rom,'BPP',2);

    cbFPS.Checked:=ini.ReadBool(Rom,'FPS',true);
    cbSkip.Checked:=ini.ReadBool(Rom,'Skip',false);
    cbDetect.Checked:=ini.ReadBool(Rom,'Detection',true);
    cbLimit.Checked:=ini.ReadBool(Rom,'FrameLimit',true);
    edFRate.text:=ini.ReadString(rom,'FrameRate','100');
    
    cbTFiltering.ItemIndex:=ini.ReadInteger(rom,'Filtering',0);
    cbTCaching.ItemIndex:=ini.ReadInteger(rom,'Caching',2);
    cbTQuality.Itemindex:=ini.ReadInteger(rom,'Quality',3);

    cbDisplay.Itemindex:=ini.ReadInteger(rom,'Display',0);
    cbBlending.Itemindex:=ini.ReadInteger(rom,'Blending',0);
    cbScanlines.Itemindex:=ini.ReadInteger(rom,'Scanlines',0);
  end;
  edDir.Text:=ini.ReadString('General','Path','Roms\');
 finally
  ini.destroy;
 end;
end;

procedure TfrmProperties.SaveSettings(Nr:Integer);
var ini:Tinifile;
    rom:String;
begin
 try
  ini:=TInifile.Create(extractfilepath(Application.ExeName)+'\'+configName);
  rom:='Rom'+inttostr(nr);

  ini.WriteString(Rom,'CutOff',edCutOff.text);
  ini.WriteString(Rom,'CBuffer',edCBuffer.text);
  ini.WriteString(Rom,'DBuffer',edDBuffer.text);
  ini.WriteBool(Rom,'Interpreter',cbInterpreter.Checked);
  ini.WriteBool(Rom,'Geometry',cbGeometry.Checked);
  ini.WriteBool(Rom,'Sound',cbSound.Checked);
  ini.WriteBool(Rom,'Filter',cbSoundFilter.checked);
  ini.WriteBool(Rom,'Surround',cbSurround.checked);
  ini.WriteBool(Rom,'Stereo',cbStereo.Checked);
  ini.WriteString(Rom,'Multiplier',edSurrMult.Text);
  ini.WriteInteger(Rom,'Renderer',cbRenderer.ItemIndex);
  ini.WriteString('General','Path',edDir.Text);

   // Renderer
    ini.WriteBool(Rom,'Windowed',cbWindow.Checked);
    ini.WriteString(Rom,'Xmax',edXmax.Text);
    ini.WriteString(Rom,'Ymax',edYmax.Text);
    ini.WriteBool(Rom,'Dithering',cbDithering.Checked);
    ini.WriteInteger(rom,'BPP',cbBpp.ItemIndex);

    ini.WriteBool(Rom,'FPS',cbFPS.Checked);
    ini.WriteBool(Rom,'Skip',cbSkip.Checked);
    ini.WriteBool(Rom,'Detection',cbDetect.Checked);
    ini.WriteBool(Rom,'FrameLimit',cbLimit.Checked);
    ini.WriteString(rom,'FrameRate',edFRate.text);

    ini.WriteInteger(rom,'Filtering',cbTFiltering.ItemIndex);
    ini.WriteInteger(rom,'Caching',cbTCaching.ItemIndex);
    ini.WriteInteger(rom,'Quality',cbTQuality.Itemindex);

    ini.WriteInteger(rom,'Display',cbDisplay.Itemindex);
    ini.WriteInteger(rom,'Blending',cbBlending.Itemindex);
    ini.WriteInteger(rom,'Scanlines',cbScanlines.Itemindex);
 finally
  ini.destroy;
 end;
end;

procedure TfrmProperties.FormCreate(Sender: TObject);
var dir:string;
    sr:TSearchrec;
    doserror:integer;
    drv:TDriver;
begin
 cbRenderer.Clear;
 dlltype('controller.znc');
 dir:=extractfilepath(Application.ExeName);

 doserror:=findfirst(dir+'*.znc',faArchive,sr);
 while doserror=0 do begin
  drv:=DLLType(dir+sr.name);
  if drv=drvRenderer then cbRenderer.Items.add(sr.name);
  doserror:=findnext(sr);
 end;
 findclose(sr);
 cbRenderer.itemindex:=0;
end;

procedure TfrmProperties.Convert(Input:String;Output:String='renderer.znc');
var fin,fout:file;
    p:pointer;
begin
 if pos('\',output)>0 then output:=extractfilepath(input)+'\'+output;
 if not fileexists(input) then exit;

 assignfile(fin,input);reset(fin,1);
 getmem(p,filesize(fin));
 assignfile(fout,output);rewrite(fout,1);
 blockread(fin,p^,filesize(fin));
 blockwrite(fout,p^,filesize(fin));
 freemem(p,filesize(fin));
 closefile(fin);closefile(fout);
end;

procedure TfrmProperties.SpeedButton1Click(Sender: TObject);
begin
 pmResolution.Popup(left+speedbutton1.left,top+speedbutton1.top);
end;

procedure TfrmProperties.N320x2001Click(Sender: TObject);
begin
 edXmax.text:='320';
 edYMax.text:='200';
end;

procedure TfrmProperties.N640x4801Click(Sender: TObject);
begin
 edXmax.text:='640';
 edYMax.text:='480';
end;

procedure TfrmProperties.N800x6001Click(Sender: TObject);
begin
 edXmax.text:='800';
 edYMax.text:='600';
end;

procedure TfrmProperties.N1024x7681Click(Sender: TObject);
begin
 edXmax.text:='1024';
 edYMax.text:='768';
end;

procedure TfrmProperties.N1400x12001Click(Sender: TObject);
begin
 edXmax.text:='1400';
 edYMax.text:='1200';
end;

procedure TfrmProperties.CreateConfig;// could be buggy:!
var f:textfile;
    fname:String;
const cdepth:Array[0..2] of byte=(15,16,32);
begin
 fname:=extractfilepath(Application.ExeName)+'\renderer.cfg';
 assignfile(f,fname);rewrite(f);

 writeln(f,'XSize=',edXmax.text);
 writeln(f,'YSize=',edYmax.text);
 writeln(f,'FullScreen=',byte(not cbWindow.checked));
 writeln(f,'ColorDepth=',cdepth[cbBpp.itemindex]);

 writeln(f,'ScanLines=',cbScanlines.text);
 writeln(f,'Filtering=',cbTFiltering.text);
 writeln(f,'Blending=',cbBlending.text);
 writeln(f,'Dithering=',byte(cbdithering.checked));
 writeln(f,'ShowFPS=',byte(cbfps.checked));

 writeln(f,'FrameLimitation=',byte(cbLimit.checked));
 writeln(f,'FrameSkipping=',byte(cbSkip.checked));
 writeln(f,'FramerateDetection=',byte(cbDetect.checked));
 writeln(f,'FramerateManual=',edFRate.text);

 writeln(f,'TextureType=',cbTQuality.itemindex);
 writeln(f,'TextureCaching=',cbTCaching.text);
 writeln(f,'TurnDisplay=',cbDisplay.text);

 closefile(f);
end;

procedure TfrmProperties.editorKeyPress(Sender: TObject; var Key: Char);
begin
 if not (key in [#13,#8,'0'..'9']) then key:=#0;
end;

end.
