unit UZipList;

interface

uses classes;

type TItem = packed record
      FileName       : String;
      PathName       : String;
      DateTime       : Integer;
      CompresSize    : Cardinal;
      UncompresSize  : Cardinal;
     end;
    TItemList = array of TItem;

    TZip=class
      private
       fItems:TItemList;
       function GetCount:Integer;
      public
       property Items:TItemList read fItems;
       property Count:Integer read GetCount;

       constructor create(Filename:String);
       destructor destroy;
      end;

implementation

uses Windows, SysUtils;

type ZipDirEntry = packed record
  VersionMade    : Word;
  VersionNeed    : Word;
  Flag           : Word;
  CompresMethod  : Word;
  DateTime       : Integer;
  CRC32          : Integer;
  CompresSize    : Cardinal;
  UncompresSize  : Cardinal;
  FileNameLen    : Word;
  ExtraFldLen    : Word;
  FileCommentLen : Word;
  StartOnDisk    : Word;
  IntFileAttrib  : Word;
  ExtFileAttrib  : Integer;
  RelOffLocalHdr : Cardinal;
 end;

 ZipEndRecord = packed record // 'PK56'
  DiskNo         : Word;
  CentralDiskNo  : Word;
  CentralEntries : Word;
  TotalEntries   : Word;
  CentralSize    : Integer;
  CentralOffSet  : Cardinal;
  ZipCommentLen  : Word;
 end;

const LocalFileHeaderSig   = $04034B50;   // 'PK34'
      CentralFileHeaderSig = $02014B50;   // 'PK12'
      EndCentralDirSig     = $06054B50;   // 'PK56'
      ExtLocalSig          = $08074B50;   // 'PK78'

constructor TZip.create(Filename:String);
var ZipStream: TFileStream;
    ZipRecord: ZipDirEntry;
    EndRecord: ZipEndRecord;
    cBuf: array[0..255]of Char;
    nPos: LongInt; nSign, idx: Integer; nFlag: Byte;
    bEndFound: Boolean;
begin
 SetLength(fItems,0);
 if not FileExists(Filename) then Exit;
 ZipStream:=TFileStream.Create(Filename,fmOpenRead or fmShareDenyNone);
 try
  if ZipStream.Size>22 then begin
   nPos:=ZipStream.Seek(-20,soFromEnd);
   bEndFound:=false;
   while (not bEndFound) and (nPos>0) and (nPos+1024>ZipStream.Size) do begin
    nPos:=ZipStream.Seek(-2,soFromCurrent);
    ZipStream.Read(nFlag,1);
    if nFlag=Ord('P') then begin
     nPos:=ZipStream.Seek(-1,soFromCurrent);
     ZipStream.Read(nSign,4);
     if nSign=EndCentralDirSig then bEndFound:=true
     else ZipStream.Seek(-4,soFromCurrent);
    end;
   end;
   if  bEndFound then begin
    ZipStream.Read(EndRecord,SizeOf(EndRecord));
    ZipStream.Seek(EndRecord.CentralOffSet,soFromBeginning);
    idx:=0;
    repeat
     ZipStream.Read(nSign,4);
     if nSign=CentralFileHeaderSig then begin
      ZipStream.Read(ZipRecord,SizeOf(ZipRecord));
      ZipStream.Read(cBuf,ZipRecord.FileNameLen);
      cBuf[ZipRecord.FileNameLen]:=#0;
      OemToChar(cBuf,cBuf);

      while StrScan(cBuf,'/')<>nil do StrScan(cBuf,'/')^:='\';

      ZipStream.Seek(ZipRecord.FileCommentLen,soFromCurrent);
      SetLength(fItems,idx+1);
      fItems[idx].DateTime:=ZipRecord.DateTime;
      fItems[idx].CompresSize:=ZipRecord.CompresSize;
      fItems[idx].UncompresSize:=ZipRecord.UncompresSize;
      fItems[idx].FileName:=ExtractFileName(cBuf);
      fItems[idx].PathName:=ExtractFilePath(cBuf);
      Inc(idx);
     end;
    until nSign<>CentralFileHeaderSig;
   end;
  end;
 finally
  ZipStream.Free;
 end;
end;

destructor TZip.destroy;
begin
 setlength(fItems,0);
end;

function TZip.GetCount:Integer;
begin
 result:=high(fItems);
end;

end.
